#ifndef UGDK_BASE_RESOURCEMANAGER_H_
#define UGDK_BASE_RESOURCEMANAGER_H_

#include <typeinfo>
#include <map>
#include <ugdk/action.h>
#include <ugdk/base/resourcecontainer.h>
#include <ugdk/graphic.h>
#include <ugdk/util.h>

#define RESOURCE_MANAGER()  (ugdk::Engine::reference()->resource_manager())

namespace ugdk {
namespace base {

class ResourceManager {
  public:
    ResourceManager();
    ~ResourceManager();
    
    static graphic::Texture*     GetTextureFromTag        (const std::string& tag);
    static graphic::Texture*     GetTextureFromFile       (const std::string& file);
    static graphic::Spritesheet* GetSpritesheetFromTag    (const std::string& tag);
    static action::AnimationSet*         GetAnimationSetFromFile  (const std::string& file);
    static graphic::Text*        CreateTextFromLanguageTag(const std::string& tag);

    // Generic Methods
    template <class T>
    void add_container(ResourceContainer<T>* container) {
        containers_[&typeid(T)] = container;
    }

    template <class T>
    ResourceContainer<T>& get_container() {
        ResourceContainerBase* base = containers_[&typeid(T)];
        ResourceContainer<T>* container = static_cast<ResourceContainer<T>*>(base);
        return *container;
    }

    // Retro-compatibility
    ResourceContainer<graphic::Texture*>&     texture_container()     { return get_container<graphic::Texture*>();     }
    ResourceContainer<graphic::Spritesheet*>& spritesheet_container() { return get_container<graphic::Spritesheet*>(); }
    ResourceContainer<action::AnimationSet*>&         animation_loader()      { return get_container<action::AnimationSet*>();         }
    ResourceContainer<LanguageWord*>&         word_container()        { return get_container<LanguageWord*>();         }
    
  private:
    typedef std::map<const std::type_info*, ResourceContainerBase*, 
        bool (*)(const std::type_info*, const std::type_info*) > ResourceMap;
    ResourceMap containers_;
};

} // namespace base
} // namespace ugdk

#endif // UGDK_BASE_RESOURCEMANAGER_H_

